# Generated by Django 3.2.18 on 2023-03-11 10:08

from django.db import migrations, models
import django.db.models.deletion
import pulp_maven.app.models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0098_pulp_labels'),
        ('maven', '0004_swap_distribution_models'),
    ]

    operations = [
        migrations.CreateModel(
            name='MavenMetadata',
            fields=[
                ('content_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='maven_mavenmetadata', serialize=False, to='core.content')),
                ('group_id', models.CharField(max_length=255)),
                ('artifact_id', models.CharField(max_length=255)),
                ('version', models.CharField(max_length=255, null=True)),
                ('filename', models.CharField(max_length=255)),
                ('sha256', models.CharField(db_index=True, max_length=64, unique=True)),
            ],
            options={
                'default_related_name': '%(app_label)s_%(model_name)s',
                'unique_together': {('group_id', 'artifact_id', 'version', 'filename', 'sha256')},
            },
            bases=(pulp_maven.app.models.MavenContentMixin, 'core.content'),
        ),
    ]
